@@ -197,5 +197,5 @@ EVP_PKEY *EVP_PKEY_dup(EVP_PKEY *pkey)
 
 int AC_verify(X509_ALGOR *algor1, ASN1_BIT_STRING *signature,char *data, EVP_PKEY *pkey)
 {
-  return ASN1_verify((int (*)())i2d_AC_INFO, algor1, signature, data, pkey);
+  return ASN1_verify((i2d_of_void *)i2d_AC_INFO, algor1, signature, data, pkey);
 }
