@@ -27,39 +27,43 @@
 
 #include "voms_api.h"
 
-/*Interface routines from C++ API to C API */
+/* Interface routines from C++ API to C API */
+
 extern "C" {
 #include "cinterface.h"
 #include <stdlib.h>
 #include <time.h>
+#include <listfunc.h>
 }
 
 #include "realdata.h"
+#include "data.h"
 
 #include <cstring>
 #include <cstdlib>
 
-#if 1
-int TranslateVOMS(struct vomsdatar *vd, std::vector<voms> &v, int *error)
+void VOMS_Destroy(struct vomsdatar *vd);
+
+int TranslateVOMS(struct vomsdatar *vd, std::vector<voms> &v, UNUSED(int *error))
 {
   if (vd->data) {
     /* Delete old store */
     free(vd->data[0]);
     free(vd->data);
   }
-  int vsize = v.size();
-  
-  struct vomsr **arr2 = (struct vomsr **)malloc((vsize+1) * sizeof(struct vomsr *));
-  int i = 0;
+
+  struct vomsr **arr2 = (struct vomsr **)malloc((v.size()+1) * sizeof(struct vomsr *));
 
   if (arr2) {
     std::vector<voms>::iterator cur = v.begin();
-    while (cur != v.end()) {
+    std::vector<voms>::iterator end = v.end();
+    int i = 0;
+    while (cur != end) {
       arr2[i] = cur->translate();
       arr2[i]->mydata = i;
       arr2[i]->my2    = (void *)vd;
       i++;
-      cur++;
+      ++cur;
     }
     arr2[i] = NULL;
     
@@ -69,7 +73,6 @@ int TranslateVOMS(struct vomsdatar *vd, std::vector<voms> &v, int *error)
   free(arr2);
   return 0;
 }
-#endif
 
 static char *
 mystrdup(const char *str, int len = 0)
@@ -95,28 +98,32 @@ struct vomsdatar *VOMS_Init(char *voms, char *cert)
 {
   struct vomsdatar *vd = NULL;
   try {
-    vomsdata *v = new vomsdata((voms ? std::string(voms) : ""),
-                               (cert ? std::string(cert) : ""));
-
     if ((vd = (struct vomsdatar *)malloc(sizeof(struct vomsdatar)))) {
       vd->cdir = mystrdup(voms ? voms : "");
       vd->vdir = mystrdup(cert ? cert : "");
       vd->data = NULL;
       vd->extra_data = vd->workvo = NULL;
       vd->volen = vd->extralen = 0;
-      vd->timeout = 30;
-      vd->real = v;
+      vd->real = new vomsdata((voms ? std::string(voms) : ""),
+                              (cert ? std::string(cert) : ""));;
+      vd->timeout = -1;
     }
   }
-  catch(...) {}
+  catch(...) {
+    goto err;
+  }
   
   return vd;
+
+ err:
+  VOMS_Destroy(vd);
+  return NULL;
 }
 
 #define GetPointer(v) (((struct realdata *)(((struct vomsdatar *)((v)->my2))->real->data[v->mydata].realdata)))
 #define GetV(v) (((struct vomsdatar *)((v)->my2))->real->data[v->mydata])
 
-int VOMS_GetAttributeSourcesNumber(struct vomsr *v, struct vomsdatar *vd, int *error)
+int VOMS_GetAttributeSourcesNumber(struct vomsr *v, UNUSED(struct vomsdatar *vd), int *error)
 {
   try {
     return GetV(v).GetAttributes().size();
@@ -139,7 +146,7 @@ int VOMS_GetAttributeSourceHandle(struct vomsr *v, int num, struct vomsdatar *vd
   return -1;
 }
 
-const char *VOMS_GetAttributeGrantor(struct vomsr *v, int handle, struct vomsdatar *vd, int *error)
+const char *VOMS_GetAttributeGrantor(struct vomsr *v, int handle, UNUSED(struct vomsdatar *vd), int *error)
 {
   try {
     return ((GetV(v).GetAttributes())[handle].grantor.c_str());
@@ -150,7 +157,7 @@ const char *VOMS_GetAttributeGrantor(struct vomsr *v, int handle, struct vomsdat
   }
 }
 
-int VOMS_GetAttributesNumber(struct vomsr *v, int handle, struct vomsdatar *vd, int *error)
+int VOMS_GetAttributesNumber(struct vomsr *v, int handle, UNUSED(struct vomsdatar *vd), int *error)
 {
   try {
     return ((GetV(v).GetAttributes())[handle].attributes.size());
@@ -161,7 +168,7 @@ int VOMS_GetAttributesNumber(struct vomsr *v, int handle, struct vomsdatar *vd,
   }
 }
 
-int VOMS_GetAttribute(struct vomsr *v, int handle, int num, struct attributer *at, struct vomsdatar *vd, int *error)
+int VOMS_GetAttribute(struct vomsr *v, int handle, int num, struct attributer *at, UNUSED(struct vomsdatar *vd), int *error)
 {
   try {
     struct attribute a = ((GetV(v).GetAttributes())[handle]).attributes[num];
@@ -190,7 +197,7 @@ static struct contactdatar **Arrayize(std::vector<contactdata> &cd, int *error)
   struct contactdatar *cda = (struct contactdatar *)malloc(size1);
 
   if (cdr && cda) {
-    std::vector<contactdata>::iterator cur = cd.begin(),
+    std::vector<contactdata>::const_iterator cur = cd.begin(),
       end = cd.end();
 
     int i = 0;
@@ -204,7 +211,7 @@ static struct contactdatar **Arrayize(std::vector<contactdata> &cd, int *error)
       cda[i].port    = cur->port;
       cda[i].version = cur->version;
       i++;
-      cur++;
+      ++cur;
     }
     cdr[i] = NULL;
     
@@ -212,7 +219,7 @@ static struct contactdatar **Arrayize(std::vector<contactdata> &cd, int *error)
   }
   else {
     *error = VERR_MEM;
-    free (cdr);
+    free(cdr);
     free(cda);
     return NULL;
   }
@@ -296,24 +303,22 @@ struct vomsr *voms::translate()
       dst->ac     = AC_dup((((struct realdata *)realdata)->ac));
       dst->holder = X509_dup(holder);
 
-      if (!dst->holder || !dst->ac)
+      if ((!dst->holder && holder) || !dst->ac)
+        throw 3;
+
+      dst->fqan = vectoarray(fqan);
+      if (!dst->fqan)
         throw 3;
 
-      dst->fqan = (char **)calloc(1, sizeof(char *)*(fqan.size()+1));
       dst->std  = (struct datar **)calloc(1, sizeof(struct datar *)*(std.size()+1));
-      if (!dst->fqan || !dst->std)
+      if (!dst->std)
         throw 3;
 
       int j = 0;
 
-      for (std::vector<std::string>::iterator i = fqan.begin();
-           i != fqan.end(); i++)
-        if (!(dst->fqan[j++] = mystrdup((*i).c_str())))
-          throw 3;
-    
-      j = 0;
-      for (std::vector<data>::iterator i = std.begin();
-             i != std.end(); i++) {
+      std::vector<data>::const_iterator end = std.end();
+      for (std::vector<data>::const_iterator i = std.begin();
+             i != end; ++i) {
         struct datar *d = (struct datar *)calloc(1, sizeof(struct datar));
         if (d) {
           dst->std[j++] = d;
@@ -335,26 +340,22 @@ struct vomsr *voms::translate()
   return NULL;
 }
 
+
+static void freeDatar(struct datar *dr)
+{
+  if (dr){
+    free(dr->group);
+    free(dr->role);
+    free(dr->cap);
+    free(dr);
+  }
+}
+
 void VOMS_Delete(struct vomsr *v) 
 {
   if (v) {
-    if (v->fqan) {
-      int i = 0;
-      while(v->fqan[i])
-        free(v->fqan[i++]);
-      free(v->fqan);
-    }
-    if (v->std) {
-      int i = 0;
-      while(v->std[i]) {
-        free(v->std[i]->group);
-        free(v->std[i]->role);
-        free(v->std[i]->cap);
-        free(v->std[i++]);
-      }
-      free (v->std);
-    }
-
+    listfree(v->fqan, free);
+    listfree((char**)v->std, (freefn)freeDatar);
     free(v->signature);
     free(v->user);
     free(v->userca);
@@ -392,17 +393,23 @@ void VOMS_Destroy(struct vomsdatar *vd)
     free(vd->vdir);
     free(vd->workvo);
     free(vd->extra_data);
-    if (vd->data) {
-      int i = 0;
-      while (vd->data[i])
-        VOMS_Delete(vd->data[i++]);
-    }
-    free(vd->data);
+    listfree((char**)vd->data, (freefn)VOMS_Delete);
     delete vd->real;
     free(vd);
   }
 }
 
+int VOMS_LoadCredentials(X509 *cert, EVP_PKEY *pkey, STACK_OF(X509) *chain, struct vomsdatar *vd, int *error)
+{
+  if (!vd || !vd->real || !error) {
+    *error = VERR_PARAM;
+    return 0;
+  }
+
+  vomsdata *v = vd->real;
+
+  return v->LoadCredentials(cert, pkey, chain) ? 1 : 0;
+}
 
 int VOMS_AddTarget(struct vomsdatar *vd, char *target, int *error)
 {
@@ -442,14 +449,16 @@ char *VOMS_ListTargets(struct vomsdatar *vd, int *error)
 
   std::vector<std::string> list = v->ListTargets();
 
-  std::vector<std::string>::iterator cur = list.begin();
-
+  std::vector<std::string>::const_iterator cur = list.begin();
+  std::vector<std::string>::const_iterator begin = list.begin();
+  std::vector<std::string>::const_iterator end = list.end();
   std::string total = "";
 
-  while(cur != list.end()) {
-    if (cur != list.begin())
+  while(cur != end) {
+    if (cur != begin)
       total += ",";
     total += *cur;
+    ++cur;
   }
 
   char *res = mystrdup(total.c_str());
@@ -652,17 +661,9 @@ int VOMS_RetrieveFromCred(gss_cred_id_t cred, int how, struct vomsdatar *vd, int
 
 int VOMS_RetrieveFromCtx(gss_ctx_id_t ctx, int how, struct vomsdatar *vd, int *error)
 {
-  if (!vd || !vd->real || !error) {
-    *error = VERR_PARAM;
-    return 0;
-  }
-  
-  vomsdata *v = vd->real;
+  if (error)
+    *error = VERR_NOTAVAIL;
 
-  if (v->RetrieveFromCtx(ctx, recurse_type(how)))
-    return TranslateVOMS(vd, v->data, error);
-
-  *error = v->error;
   return 0;
 }
 
@@ -682,6 +683,22 @@ int VOMS_RetrieveFromProxy(int how, struct vomsdatar *vd, int *error)
   return 0;
 }
 
+int VOMS_RetrieveFromAC(AC *ac, struct vomsdatar *vd, int *error)
+{
+  if (!vd || !vd->real || !error) {
+    *error = VERR_PARAM;
+    return 0;
+  }
+
+  vomsdata *v = vd->real;
+
+  if (v->Retrieve(ac))
+    return TranslateVOMS(vd, v->data, error);
+
+  *error = v->error;
+  return 0;
+}
+
 int VOMS_Import(char *buffer, int buflen, struct vomsdatar *vd, int *error)
 {
   if (!buffer || !buflen || !vd || !vd->real || !error) {
@@ -865,8 +882,6 @@ vomsdatar *VOMS_Duplicate(vomsdatar *orig)
   struct vomsdatar *vd = NULL;
 
   try {
-    vomsdata *v = new vomsdata(*(orig->real));
-
     if ((vd = (struct vomsdatar *)malloc(sizeof(struct vomsdatar)))) {
       int error = 0;
 
@@ -878,17 +893,21 @@ vomsdatar *VOMS_Duplicate(vomsdatar *orig)
       vd->volen = orig->volen;
       vd->extralen = orig->extralen;
       vd->timeout = orig->timeout;
-      vd->real = v;
+      vd->real = new vomsdata(*(orig->real));
 
-      if (!TranslateVOMS(vd, v->data, &error)) {
-        VOMS_Destroy(vd);
-        vd = NULL;
-      }
+      if (!TranslateVOMS(vd, vd->real->data, &error))
+        goto err;
     }
   }
-  catch(...) {}
+  catch(...) {
+    goto err;
+  }
 
   return vd;
+
+ err:
+  VOMS_Destroy(vd);
+  return NULL;
 }
 
 AC *VOMS_GetAC(vomsr *v)
@@ -906,45 +925,13 @@ char **VOMS_GetTargetsList(struct vomsr *v, struct vomsdatar *vd, int *error)
 
   std::vector<std::string> targets = GetV(v).GetTargets();
 
-  int size = targets.size();
-
-  char **array = (char **)malloc(sizeof(char*)*(size+1));
-
-  if (array) {
-    int i = 0;
-    for (i = 0; i < size; i++) {
-      array[i] = mystrdup(targets[i].c_str());
-      if (!array[i])
-        goto err;
-    }
-    array[i] = NULL;
-    
-    return array;
-
-  }
-
-err:
-  if (array) {
-    int j = 0;
-
-    while (array[j])
-      free(array[j++]);
-  }
-  free(array);
-
-  return NULL;
+  return vectoarray(targets);
 }
 
 
 void VOMS_FreeTargetsList(char **targets)
 {
-  if (targets) {
-    int j = 0;
-    while (targets[j])
-      free(targets[j++]);
-  }
-
-  free(targets);
+  listfree(targets, free);
 }
 
 }
