@@ -28,6 +28,7 @@
 
 #include <string>
 #include <vector>
+#include <ostream>
 #include <exception>
 #include "voms_api.h"
 
@@ -40,7 +41,7 @@ extern "C" {
   
 }
 
-
+enum message_type {FORCED, INFO, WARN, ERROR, DEBUG};
 
 class VOMSException : public std::exception 
 {
@@ -104,11 +105,10 @@ class Client {
   std::string              targetlist;
   std::vector<std::string> confiles;
 #ifdef CLASS_ADD
-  void *                   class_add_buf = NULL;
-  size_t                   class_add_buf_len = 0;
+  void *                   class_add_buf;
+  size_t                   class_add_buf_len;
 #endif
 
-  BIGNUM *                 dataorder;
   //  proxy_cred_desc *        pcd;
   proxy_verify_desc        pvd;
   proxy_verify_ctx_desc    pvxd;
@@ -124,36 +124,42 @@ class Client {
   X509                     *ucert;
   EVP_PKEY                 *private_key;
   int                       timeout;
+  std::string               acfile;
+  vomsdata                 *v;
+
  public:
   
   Client(int argc, char** argv);
   ~Client();
-  bool Run();
+  int Run();
 
  private:
   
-  bool CreateProxy(std::string data, std::string filedata, AC ** aclist, int version);
-  X509_EXTENSION * CreateProxyExtension(std::string name, std::string data, bool crit = false);
+  bool CreateProxy(std::string data, AC ** aclist, int version);
 
-  bool Retrieve(AC *ac);
+  bool AddToList(AC *ac);
   
   // write AC and data retrieved form server to file
   bool WriteSeparate();
   
-  // include a file in a non critical extension
-  bool IncludeFile(std::string& filedata);
-  
   // test if certificate used for signing is expired
   bool Test();
   
   bool pcdInit();
   
   // verify the certificate is signed by a trusted CA
-  bool Verify(bool doproxy);
+  int Verify();
   
   // get openssl error */
   void Error();
+  void CleanAll();
 
-  bool LoadVomses(vomsdata &, bool);
+  bool LoadVomses();
+  std::ostream& Print(message_type type);
+  bool checkstats(char *file, int mode);
+  void PrintProxyCreationError(int, void *);
+  AC *ReadSeparate(const std::string&);
+  void PrintConnectResult(int status, const std::string& contact);
+  void parse_time(const std::string& timespec, const std::string& option, int& hours, int& minutes);
 };
 #endif
