@@ -36,14 +36,12 @@
 #include <string>
 
 const std::string SUBPACKAGE      = "voms-proxy-destroy";
-//const std::string VERSION         = "0.1";
 
 extern "C" {
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-  //#include <getopt.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/mman.h>
@@ -83,11 +81,11 @@ int
 main(int argc, char **argv)
 {
     if (strrchr(argv[0],'/'))
-	program = strrchr(argv[0],'/') + 1;
+      program = strrchr(argv[0],'/') + 1;
     else
-	program = argv[0];
+      program = argv[0];
 
-    static char *LONG_USAGE = \
+    static std::string LONG_USAGE = \
       "\n" \
       "    Options\n" \
       "    -help, -usage       Displays usage\n" \
@@ -135,8 +133,7 @@ main(int argc, char **argv)
 static bool
 delete_proxy(void)
 {
-  char *ccaf, *cd, *of, *cf, *kf;
-  proxy_cred_desc *pcd;
+  char *of;
 
 #ifdef WIN32
   CRYPTO_malloc_init();
@@ -145,59 +142,75 @@ delete_proxy(void)
   ERR_load_prxyerr_strings(0);
   SSLeay_add_ssl_algorithms();
 
-  if ((pcd = proxy_cred_desc_new()) == NULL)
-    return 0;
-
-  pcd->type = CRED_TYPE_PERMANENT;
-
   /*
    * These 5 const_cast are allowed because proxy_get_filenames will
    * overwrite the pointers, not the data itself.
    */
-  ccaf = NULL;
-  cd   = NULL;
-  of   = (file.empty() ? NULL : const_cast<char *>(file.c_str()));
-  cf   = NULL;
-  kf   = NULL;
+  of = (file.empty() ? NULL : strdup(const_cast<char *>(file.c_str())));
     
-  if (!determine_filenames(&ccaf, &cd, &of, &cf, &kf, 0))
+  if (!determine_filenames(NULL, NULL, &of, NULL, NULL, 0)) {
+    free(of);
     return 0;
+  }
 
-  return destroy_proxy(of, dryrun);
+  int ret = destroy_proxy(of, dryrun);
+
+  free(of);
+
+  return ret;
+}
+
+static int real_write(int fd, char *buffer, int size)
+{
+  int written = 0;
+  int current = 0;
+
+  do {
+    written = write(fd, buffer + current, size - current);
+    if (written >= 0) {
+      current += written;
+    }
+  } while ((written > 0) && (current != size));
+
+  return (current == size ? size : -1);
 }
 
 int destroy_proxy(char *file, bool dry)
 {
-  char *delblock[100];
-  
-  if (dry)
-    std::cerr << "Would remove " << file << std::endl;
-  else {
-    for (int i = 0; i <100; i++)
-      delblock[i] = '\0';
+  char delblock[100];
+
+  int fd = open(file, O_RDWR);
+
+  if (fd != -1) {
+    if (dry) {
+      if (!quiet)
+        std::cerr << "Would remove " << file << std::endl;
+      close(fd);
+    }
+    else {
+      memset(delblock, 0, 100);
 
-    int fd = open(file, O_RDWR);
-    if (fd != -1) {
       int size = lseek(fd, 0L, SEEK_END);
       lseek(fd, 0L, SEEK_SET);
       if (size > 0) {
         int num = size / 100;
         int rem = size % 100;
 
-        while (num--) 
-          write(fd,delblock,100);
+        while (num--)
+          (void)real_write(fd, delblock, 100);
+
         if (rem)
-          write(fd, delblock, rem);
+          (void)real_write(fd, delblock, rem);
       }
       close(fd);
       remove(file);
     }
-    else {
-      printf("\nProxy file doesn't exist or has bad permissions\n\n");
-      return 1;
-    }
-    
-    return 0;
   }
+  else {
+    if (!quiet)
+      std::cerr << "\nProxy file doesn't exist or has bad permissions\n" << std::endl;
+    return 1;
+  }
+
   return 0;
 }
