@@ -109,9 +109,9 @@ bool quiet = false;
 
 extern "C" {
   
-static int (*pw_cb)() = NULL;
+static pem_password_cb *pw_cb = NULL;
 
-static int pwstdin_callback(char * buf, int num, UNUSED(int w)) 
+static int pwstdin_callback(char * buf, int num, UNUSED(int w), UNUSED(void *u))
 {
   int i;
   
@@ -431,7 +431,7 @@ Fake::Fake(int argc, char ** argv) :   confile(conf_file_name),
   /* allow password from stdin */
 
   if(pwstdin)
-    pw_cb = (int (*)())(pwstdin_callback);
+    pw_cb = pwstdin_callback;
 
   /* with --debug prints configuration files used */
 
